//+---------------------------------------------------------------------------
//
//  Microsoft Windows
//  Copyright (C) Microsoft Corporation, 1992 - 1996.
//
//  File:       oleauto.h
//
//  Contents:   Defines Ole Automation support function prototypes, constants
//
//----------------------------------------------------------------------------

#if !defined( _OLEAUTO_H_ )
#define _OLEAUTO_H_

// Set packing to 8 for ISV, and Win95 support
#ifndef RC_INVOKED
#include <pshpack8.h>
#endif // RC_INVOKED

//  Definition of the OLE Automation APIs, and macros.

#ifdef _OLEAUT32_
#define WINOLEAUTAPI        STDAPI
#define WINOLEAUTAPI_(type) STDAPI_(type)
#else
#define WINOLEAUTAPI        EXTERN_C DECLSPEC_IMPORT HRESULT STDAPICALLTYPE
#define WINOLEAUTAPI_(type) EXTERN_C DECLSPEC_IMPORT type STDAPICALLTYPE
#endif

#define STDOLE_MAJORVERNUM  0x1
#define STDOLE_MINORVERNUM  0x0
#define STDOLE_LCID         0x0000

// Version # of stdole2.tlb
#define STDOLE2_MAJORVERNUM 0x2
#define STDOLE2_MINORVERNUM 0x0
#define STDOLE2_LCID        0x0000

/* if not already picked up from olenls.h */
#ifndef _LCID_DEFINED
typedef DWORD LCID;
# define _LCID_DEFINED
#endif

/* pull in the MIDL generated header */
#include <oaidl.h>


/*---------------------------------------------------------------------*/
/*                            BSTR API                                 */
/*---------------------------------------------------------------------*/

STDAPI_(BSTR) SysAllocString(const OLECHAR *);
STDAPI_(INT)  SysReAllocString(BSTR *, const OLECHAR *);
STDAPI_(BSTR) SysAllocStringLen(const OLECHAR *, UINT);
STDAPI_(INT)  SysReAllocStringLen(BSTR *, const OLECHAR *, UINT);
STDAPI_(void) SysFreeString(BSTR);
STDAPI_(UINT) SysStringLen(BSTR);

#ifdef _WIN32
STDAPI_(UINT) SysStringByteLen(BSTR bstr);
STDAPI_(BSTR) SysAllocStringByteLen(LPCSTR psz, UINT len);
#endif

/*---------------------------------------------------------------------*/
/*                            Time API                                 */
/*---------------------------------------------------------------------*/

STDAPI_(INT) DosDateTimeToVariantTime(USHORT wDosDate, USHORT wDosTime, DOUBLE * pvtime);

STDAPI_(INT) VariantTimeToDosDateTime(DOUBLE vtime, USHORT * pwDosDate, USHORT * pwDosTime);

#ifdef _WIN32
STDAPI_(INT) SystemTimeToVariantTime(LPSYSTEMTIME lpSystemTime, DOUBLE *pvtime);
STDAPI_(INT) VariantTimeToSystemTime(DOUBLE vtime, LPSYSTEMTIME lpSystemTime);
#endif


/*---------------------------------------------------------------------*/
/*                          SafeArray API                              */
/*---------------------------------------------------------------------*/

STDAPI SafeArrayAllocDescriptor(UINT cDims, SAFEARRAY ** ppsaOut);
STDAPI SafeArrayAllocData(SAFEARRAY * psa);
STDAPI_(SAFEARRAY *) SafeArrayCreate(VARTYPE vt, UINT cDims, SAFEARRAYBOUND * rgsabound);
STDAPI_(SAFEARRAY *) SafeArrayCreateVector(VARTYPE vt, LONG lLbound, ULONG cElements);
STDAPI SafeArrayCopyData(SAFEARRAY *psaSource, SAFEARRAY *psaTarget);
STDAPI SafeArrayDestroyDescriptor(SAFEARRAY * psa);
STDAPI SafeArrayDestroyData(SAFEARRAY * psa);
STDAPI SafeArrayDestroy(SAFEARRAY * psa);
STDAPI SafeArrayRedim(SAFEARRAY * psa, SAFEARRAYBOUND * psaboundNew);
STDAPI_(UINT) SafeArrayGetDim(SAFEARRAY * psa);
STDAPI_(UINT) SafeArrayGetElemsize(SAFEARRAY * psa);
STDAPI SafeArrayGetUBound(SAFEARRAY * psa, UINT nDim, LONG * plUbound);
STDAPI SafeArrayGetLBound(SAFEARRAY * psa, UINT nDim, LONG * plLbound);
STDAPI SafeArrayLock(SAFEARRAY * psa);
STDAPI SafeArrayUnlock(SAFEARRAY * psa);
STDAPI SafeArrayAccessData(SAFEARRAY * psa, void HUGEP** ppvData);
STDAPI SafeArrayUnaccessData(SAFEARRAY * psa);
STDAPI SafeArrayGetElement(SAFEARRAY * psa, LONG * rgIndices, void * pv);
STDAPI SafeArrayPutElement(SAFEARRAY * psa, LONG * rgIndices, void * pv);
STDAPI SafeArrayCopy(SAFEARRAY * psa, SAFEARRAY ** ppsaOut);
STDAPI SafeArrayPtrOfIndex(SAFEARRAY * psa, LONG * rgIndices, void ** ppvData);


/*---------------------------------------------------------------------*/
/*                           VARIANT API                               */
/*---------------------------------------------------------------------*/

STDAPI_(void) VariantInit(VARIANTARG * pvarg);
STDAPI VariantClear(VARIANTARG * pvarg);
STDAPI VariantCopy(VARIANTARG * pvargDest, VARIANTARG * pvargSrc);
STDAPI VariantCopyInd(VARIANT * pvarDest, VARIANTARG * pvargSrc);
STDAPI VariantChangeType(VARIANTARG * pvargDest,
                VARIANTARG * pvarSrc, USHORT wFlags, VARTYPE vt);
STDAPI VariantChangeTypeEx(VARIANTARG * pvargDest,
                VARIANTARG * pvarSrc, LCID lcid, USHORT wFlags, VARTYPE vt);

// Flags for VariantChangeType/VariantChangeTypeEx
#define VARIANT_NOVALUEPROP 0x1
#define VARIANT_ALPHABOOL   0x2 // For VT_BOOL to VT_BSTR conversions,
                                // convert to "True"/"False" instead of
                                // "-1"/"0"


/*---------------------------------------------------------------------*/
/*                Vector <-> Bstr conversion APIs                      */
/*---------------------------------------------------------------------*/

STDAPI VectorFromBstr (BSTR bstr, SAFEARRAY ** ppsa);
STDAPI BstrFromVector (SAFEARRAY *psa, BSTR *pbstr);


/*---------------------------------------------------------------------*/
/*                     VARTYPE Coercion API                            */
/*---------------------------------------------------------------------*/

/* Note: The routines that convert *from* a string are defined
 * to take a OLECHAR* rather than a BSTR because no allocation is
 * required, and this makes the routines a bit more generic.
 * They may of course still be passed a BSTR as the strIn param.
 */

/* Any of the coersion functions that converts either from or to a string
 * takes an additional lcid and dwFlags arguments. The lcid argument allows
 * locale specific parsing to occur.  The dwFlags allow additional function
 * specific condition to occur.  All function that accept the dwFlags argument
 * can include either 0 or LOCALE_NOUSEROVERRIDE flag. In addition, the
 * VarDateFromStr functions also accepts the VAR_TIMEVALUEONLY and
 * VAR_DATEVALUEONLY flags
 */

#define VAR_TIMEVALUEONLY   0x0001    /* return time value */
#define VAR_DATEVALUEONLY   0x0002    /* return date value */

STDAPI VarUI1FromI2(SHORT sIn, BYTE * pbOut);
STDAPI VarUI1FromI4(LONG lIn, BYTE * pbOut);
STDAPI VarUI1FromR4(FLOAT fltIn, BYTE * pbOut);
STDAPI VarUI1FromR8(DOUBLE dblIn, BYTE * pbOut);
STDAPI VarUI1FromCy(CY cyIn, BYTE * pbOut);
STDAPI VarUI1FromDate(DATE dateIn, BYTE * pbOut);
STDAPI VarUI1FromStr(OLECHAR * strIn, LCID lcid, ULONG dwFlags, BYTE * pbOut);
STDAPI VarUI1FromDisp(IDispatch * pdispIn, LCID lcid, BYTE * pbOut);
STDAPI VarUI1FromBool(VARIANT_BOOL boolIn, BYTE * pbOut);
STDAPI VarUI1FromI1(CHAR cIn, BYTE *pbOut);
STDAPI VarUI1FromUI2(USHORT uiIn, BYTE *pbOut);
STDAPI VarUI1FromUI4(ULONG ulIn, BYTE *pbOut);
STDAPI VarUI1FromDec(DECIMAL *pdecIn, BYTE *pbOut);

STDAPI VarI2FromUI1(BYTE bIn, SHORT * psOut);
STDAPI VarI2FromI4(LONG lIn, SHORT * psOut);
STDAPI VarI2FromR4(FLOAT fltIn, SHORT * psOut);
STDAPI VarI2FromR8(DOUBLE dblIn, SHORT * psOut);
STDAPI VarI2FromCy(CY cyIn, SHORT * psOut);
STDAPI VarI2FromDate(DATE dateIn, SHORT * psOut);
STDAPI VarI2FromStr(OLECHAR * strIn, LCID lcid, ULONG dwFlags, SHORT * psOut);
STDAPI VarI2FromDisp(IDispatch * pdispIn, LCID lcid, SHORT * psOut);
STDAPI VarI2FromBool(VARIANT_BOOL boolIn, SHORT * psOut);
STDAPI VarI2FromI1(CHAR cIn, SHORT *psOut);
STDAPI VarI2FromUI2(USHORT uiIn, SHORT *psOut);
STDAPI VarI2FromUI4(ULONG ulIn, SHORT *psOut);
STDAPI VarI2FromDec(DECIMAL *pdecIn, SHORT *psOut);

STDAPI VarI4FromUI1(BYTE bIn, LONG * plOut);
STDAPI VarI4FromI2(SHORT sIn, LONG * plOut);
STDAPI VarI4FromR4(FLOAT fltIn, LONG * plOut);
STDAPI VarI4FromR8(DOUBLE dblIn, LONG * plOut);
STDAPI VarI4FromCy(CY cyIn, LONG * plOut);
STDAPI VarI4FromDate(DATE dateIn, LONG * plOut);
STDAPI VarI4FromStr(OLECHAR * strIn, LCID lcid, ULONG dwFlags, LONG * plOut);
STDAPI VarI4FromDisp(IDispatch * pdispIn, LCID lcid, LONG * plOut);
STDAPI VarI4FromBool(VARIANT_BOOL boolIn, LONG * plOut);
STDAPI VarI4FromI1(CHAR cIn, LONG *plOut);
STDAPI VarI4FromUI2(USHORT uiIn, LONG *plOut);
STDAPI VarI4FromUI4(ULONG ulIn, LONG *plOut);
STDAPI VarI4FromDec(DECIMAL *pdecIn, LONG *plOut);
STDAPI VarI4FromInt(INT intIn, LONG *plOut);

STDAPI VarR4FromUI1(BYTE bIn, FLOAT * pfltOut);
STDAPI VarR4FromI2(SHORT sIn, FLOAT * pfltOut);
STDAPI VarR4FromI4(LONG lIn, FLOAT * pfltOut);
STDAPI VarR4FromR8(DOUBLE dblIn, FLOAT * pfltOut);
STDAPI VarR4FromCy(CY cyIn, FLOAT * pfltOut);
STDAPI VarR4FromDate(DATE dateIn, FLOAT * pfltOut);
STDAPI VarR4FromStr(OLECHAR * strIn, LCID lcid, ULONG dwFlags, FLOAT *pfltOut);
STDAPI VarR4FromDisp(IDispatch * pdispIn, LCID lcid, FLOAT * pfltOut);
STDAPI VarR4FromBool(VARIANT_BOOL boolIn, FLOAT * pfltOut);
STDAPI VarR4FromI1(CHAR cIn, FLOAT *pfltOut);
STDAPI VarR4FromUI2(USHORT uiIn, FLOAT *pfltOut);
STDAPI VarR4FromUI4(ULONG ulIn, FLOAT *pfltOut);
STDAPI VarR4FromDec(DECIMAL *pdecIn, FLOAT *pfltOut);

STDAPI VarR8FromUI1(BYTE bIn, DOUBLE * pdblOut);
STDAPI VarR8FromI2(SHORT sIn, DOUBLE * pdblOut);
STDAPI VarR8FromI4(LONG lIn, DOUBLE * pdblOut);
STDAPI VarR8FromR4(FLOAT fltIn, DOUBLE * pdblOut);
STDAPI VarR8FromCy(CY cyIn, DOUBLE * pdblOut);
STDAPI VarR8FromDate(DATE dateIn, DOUBLE * pdblOut);
STDAPI VarR8FromStr(OLECHAR *strIn, LCID lcid, ULONG dwFlags, DOUBLE *pdblOut);
STDAPI VarR8FromDisp(IDispatch * pdispIn, LCID lcid, DOUBLE * pdblOut);
STDAPI VarR8FromBool(VARIANT_BOOL boolIn, DOUBLE * pdblOut);
STDAPI VarR8FromI1(CHAR cIn, DOUBLE *pdblOut);
STDAPI VarR8FromUI2(USHORT uiIn, DOUBLE *pdblOut);
STDAPI VarR8FromUI4(ULONG ulIn, DOUBLE *pdblOut);
STDAPI VarR8FromDec(DECIMAL *pdecIn, DOUBLE *pdblOut);

STDAPI VarDateFromUI1(BYTE bIn, DATE * pdateOut);
STDAPI VarDateFromI2(SHORT sIn, DATE * pdateOut);
STDAPI VarDateFromI4(LONG lIn, DATE * pdateOut);
STDAPI VarDateFromR4(FLOAT fltIn, DATE * pdateOut);
STDAPI VarDateFromR8(DOUBLE dblIn, DATE * pdateOut);
STDAPI VarDateFromCy(CY cyIn, DATE * pdateOut);
STDAPI VarDateFromStr(OLECHAR *strIn, LCID lcid, ULONG dwFlags, DATE *pdateOut);
STDAPI VarDateFromDisp(IDispatch * pdispIn, LCID lcid, DATE * pdateOut);
STDAPI VarDateFromBool(VARIANT_BOOL boolIn, DATE * pdateOut);
STDAPI VarDateFromI1(CHAR cIn, DATE *pdateOut);
STDAPI VarDateFromUI2(USHORT uiIn, DATE *pdateOut);
STDAPI VarDateFromUI4(ULONG ulIn, DATE *pdateOut);
STDAPI VarDateFromDec(DECIMAL *pdecIn, DATE *pdateOut);

STDAPI VarCyFromUI1(BYTE bIn, CY * pcyOut);
STDAPI VarCyFromI2(SHORT sIn, CY * pcyOut);
STDAPI VarCyFromI4(LONG lIn, CY * pcyOut);
STDAPI VarCyFromR4(FLOAT fltIn, CY * pcyOut);
STDAPI VarCyFromR8(DOUBLE dblIn, CY * pcyOut);
STDAPI VarCyFromDate(DATE dateIn, CY * pcyOut);
STDAPI VarCyFromStr(OLECHAR * strIn, LCID lcid, ULONG dwFlags, CY * pcyOut);
STDAPI VarCyFromDisp(IDispatch * pdispIn, LCID lcid, CY * pcyOut);
STDAPI VarCyFromBool(VARIANT_BOOL boolIn, CY * pcyOut);
STDAPI VarCyFromI1(CHAR cIn, CY *pcyOut);
STDAPI VarCyFromUI2(USHORT uiIn, CY *pcyOut);
STDAPI VarCyFromUI4(ULONG ulIn, CY *pcyOut);
STDAPI VarCyFromDec(DECIMAL *pdecIn, CY *pcyOut);

STDAPI VarBstrFromUI1(BYTE bVal, LCID lcid, ULONG dwFlags, BSTR * pbstrOut);
STDAPI VarBstrFromI2(SHORT iVal, LCID lcid, ULONG dwFlags, BSTR * pbstrOut);
STDAPI VarBstrFromI4(LONG lIn, LCID lcid, ULONG dwFlags, BSTR * pbstrOut);
STDAPI VarBstrFromR4(FLOAT fltIn, LCID lcid, ULONG dwFlags, BSTR * pbstrOut);
STDAPI VarBstrFromR8(DOUBLE dblIn, LCID lcid, ULONG dwFlags, BSTR * pbstrOut);
STDAPI VarBstrFromCy(CY cyIn, LCID lcid, ULONG dwFlags, BSTR * pbstrOut);
STDAPI VarBstrFromDate(DATE dateIn, LCID lcid, ULONG dwFlags, BSTR * pbstrOut);
STDAPI VarBstrFromDisp(IDispatch * pdispIn, LCID lcid, ULONG dwFlags, BSTR * pbstrOut);
STDAPI VarBstrFromBool(VARIANT_BOOL boolIn, LCID lcid, ULONG dwFlags, BSTR * pbstrOut);
STDAPI VarBstrFromI1(CHAR cIn, LCID lcid, ULONG dwFlags, BSTR *pbstrOut);
STDAPI VarBstrFromUI2(USHORT uiIn, LCID lcid, ULONG dwFlags, BSTR *pbstrOut);
STDAPI VarBstrFromUI4(ULONG ulIn, LCID lcid, ULONG dwFlags, BSTR *pbstrOut);
STDAPI VarBstrFromDec(DECIMAL *pdecIn, LCID lcid, ULONG dwFlags, BSTR *pbstrOut);

STDAPI VarBoolFromUI1(BYTE bIn, VARIANT_BOOL * pboolOut);
STDAPI VarBoolFromI2(SHORT sIn, VARIANT_BOOL * pboolOut);
STDAPI VarBoolFromI4(LONG lIn, VARIANT_BOOL * pboolOut);
STDAPI VarBoolFromR4(FLOAT fltIn, VARIANT_BOOL * pboolOut);
STDAPI VarBoolFromR8(DOUBLE dblIn, VARIANT_BOOL * pboolOut);
STDAPI VarBoolFromDate(DATE dateIn, VARIANT_BOOL * pboolOut);
STDAPI VarBoolFromCy(CY cyIn, VARIANT_BOOL * pboolOut);
STDAPI VarBoolFromStr(OLECHAR * strIn, LCID lcid, ULONG dwFlags, VARIANT_BOOL * pboolOut);
STDAPI VarBoolFromDisp(IDispatch * pdispIn, LCID lcid, VARIANT_BOOL * pboolOut);
STDAPI VarBoolFromI1(CHAR cIn, VARIANT_BOOL *pboolOut);
STDAPI VarBoolFromUI2(USHORT uiIn, VARIANT_BOOL *pboolOut);
STDAPI VarBoolFromUI4(ULONG ulIn, VARIANT_BOOL *pboolOut);
STDAPI VarBoolFromDec(DECIMAL *pdecIn, VARIANT_BOOL *pboolOut);

STDAPI VarI1FromUI1(BYTE bIn, CHAR *pcOut);
STDAPI VarI1FromI2(SHORT uiIn, CHAR *pcOut);
STDAPI VarI1FromI4(LONG lIn, CHAR *pcOut);
STDAPI VarI1FromR4(FLOAT fltIn, CHAR *pcOut);
STDAPI VarI1FromR8(DOUBLE dblIn, CHAR *pcOut);
STDAPI VarI1FromDate(DATE dateIn, CHAR *pcOut);
STDAPI VarI1FromCy(CY cyIn, CHAR *pcOut);
STDAPI VarI1FromStr(OLECHAR *strIn, LCID lcid, ULONG dwFlags, CHAR *pcOut);
STDAPI VarI1FromDisp(IDispatch *pdispIn, LCID lcid, CHAR *pcOut);
STDAPI VarI1FromBool(VARIANT_BOOL boolIn, CHAR *pcOut);
STDAPI VarI1FromUI2(USHORT uiIn, CHAR *pcOut);
STDAPI VarI1FromUI4(ULONG ulIn, CHAR *pcOut);
STDAPI VarI1FromDec(DECIMAL *pdecIn, CHAR *pcOut);

STDAPI VarUI2FromUI1(BYTE bIn, USHORT *puiOut);
STDAPI VarUI2FromI2(SHORT uiIn, USHORT *puiOut);
STDAPI VarUI2FromI4(LONG lIn, USHORT *puiOut);
STDAPI VarUI2FromR4(FLOAT fltIn, USHORT *puiOut);
STDAPI VarUI2FromR8(DOUBLE dblIn, USHORT *puiOut);
STDAPI VarUI2FromDate(DATE dateIn, USHORT *puiOut);
STDAPI VarUI2FromCy(CY cyIn, USHORT *puiOut);
STDAPI VarUI2FromStr(OLECHAR *strIn, LCID lcid, ULONG dwFlags, USHORT *puiOut);
STDAPI VarUI2FromDisp(IDispatch *pdispIn, LCID lcid, USHORT *puiOut);
STDAPI VarUI2FromBool(VARIANT_BOOL boolIn, USHORT *puiOut);
STDAPI VarUI2FromI1(CHAR cIn, USHORT *puiOut);
STDAPI VarUI2FromUI4(ULONG ulIn, USHORT *puiOut);
STDAPI VarUI2FromDec(DECIMAL *pdecIn, USHORT *puiOut);

STDAPI VarUI4FromUI1(BYTE bIn, ULONG *pulOut);
STDAPI VarUI4FromI2(SHORT uiIn, ULONG *pulOut);
STDAPI VarUI4FromI4(LONG lIn, ULONG *pulOut);
STDAPI VarUI4FromR4(FLOAT fltIn, ULONG *pulOut);
STDAPI VarUI4FromR8(DOUBLE dblIn, ULONG *pulOut);
STDAPI VarUI4FromDate(DATE dateIn, ULONG *pulOut);
STDAPI VarUI4FromCy(CY cyIn, ULONG *pulOut);
STDAPI VarUI4FromStr(OLECHAR *strIn, LCID lcid, ULONG dwFlags, ULONG *pulOut);
STDAPI VarUI4FromDisp(IDispatch *pdispIn, LCID lcid, ULONG *pulOut);
STDAPI VarUI4FromBool(VARIANT_BOOL boolIn, ULONG *pulOut);
STDAPI VarUI4FromI1(CHAR cIn, ULONG *pulOut);
STDAPI VarUI4FromUI2(USHORT uiIn, ULONG *pulOut);
STDAPI VarUI4FromDec(DECIMAL *pdecIn, ULONG *pulOut);

STDAPI VarDecFromUI1(BYTE bIn, DECIMAL *pdecOut);
STDAPI VarDecFromI2(SHORT uiIn, DECIMAL *pdecOut);
STDAPI VarDecFromI4(LONG lIn, DECIMAL *pdecOut);
STDAPI VarDecFromR4(FLOAT fltIn, DECIMAL *pdecOut);
STDAPI VarDecFromR8(DOUBLE dblIn, DECIMAL *pdecOut);
STDAPI VarDecFromDate(DATE dateIn, DECIMAL *pdecOut);
STDAPI VarDecFromCy(CY cyIn, DECIMAL *pdecOut);
STDAPI VarDecFromStr(OLECHAR *strIn, LCID lcid, ULONG dwFlags, DECIMAL *pdecOut);
STDAPI VarDecFromDisp(IDispatch *pdispIn, LCID lcid, DECIMAL *pdecOut);
STDAPI VarDecFromBool(VARIANT_BOOL boolIn, DECIMAL *pdecOut);
STDAPI VarDecFromI1(CHAR cIn, DECIMAL *pdecOut);
STDAPI VarDecFromUI2(USHORT uiIn, DECIMAL *pdecOut);
STDAPI VarDecFromUI4(ULONG ulIn, DECIMAL *pdecOut);

#define VarUI4FromUI4(in, pOut) (*(pOut) = (in))
#define VarI4FromI4(in, pOut)   (*(pOut) = (in))

#define VarUI1FromInt       VarUI1FromI4
#define VarUI1FromUint      VarUI1FromUI4
#define VarI2FromInt        VarI2FromI4
#define VarI2FromUint       VarI2FromUI4
#define VarI4FromInt        VarI4FromI4
#define VarI4FromUint       VarI4FromUI4
#define VarR4FromInt        VarR4FromI4
#define VarR4FromUint       VarR4FromUI4
#define VarR8FromInt        VarR8FromI4
#define VarR8FromUint       VarR8FromUI4
#define VarDateFromInt      VarDateFromI4
#define VarDateFromUint     VarDateFromUI4
#define VarCyFromInt        VarCyFromI4
#define VarCyFromUint       VarCyFromUI4
#define VarBstrFromInt      VarBstrFromI4
#define VarBstrFromUint     VarBstrFromUI4
#define VarBoolFromInt      VarBoolFromI4
#define VarBoolFromUint     VarBoolFromUI4
#define VarI1FromInt        VarI1FromI4
#define VarI1FromUint       VarI1FromUI4
#define VarUI2FromInt       VarUI2FromI4
#define VarUI2FromUint      VarUI2FromUI4
#define VarUI4FromInt       VarUI4FromI4
#define VarUI4FromUint      VarUI4FromUI4
#define VarDecFromInt       VarDecFromI4
#define VarDecFromUint      VarDecFromUI4
#define VarIntFromUI1       VarI4FromUI1
#define VarIntFromI2        VarI4FromI2
#define VarIntFromI4        VarI4FromI4
#define VarIntFromR4        VarI4FromR4
#define VarIntFromR8        VarI4FromR8
#define VarIntFromDate      VarI4FromDate
#define VarIntFromCy        VarI4FromCy
#define VarIntFromStr       VarI4FromStr
#define VarIntFromDisp      VarI4FromDisp
#define VarIntFromBool      VarI4FromBool
#define VarIntFromI1        VarI4FromI1
#define VarIntFromUI2       VarI4FromUI2
#define VarIntFromUI4       VarI4FromUI4
#define VarIntFromDec       VarI4FromDec
#define VarIntFromUint      VarI4FromUI4
#define VarUintFromUI1      VarUI4FromUI1
#define VarUintFromI2       VarUI4FromI2
#define VarUintFromI4       VarUI4FromI4
#define VarUintFromR4       VarUI4FromR4
#define VarUintFromR8       VarUI4FromR8
#define VarUintFromDate     VarUI4FromDate
#define VarUintFromCy       VarUI4FromCy
#define VarUintFromStr      VarUI4FromStr
#define VarUintFromDisp     VarUI4FromDisp
#define VarUintFromBool     VarUI4FromBool
#define VarUintFromI1       VarUI4FromI1
#define VarUintFromUI2      VarUI4FromUI2
#define VarUintFromUI4      VarUI4FromUI4
#define VarUintFromDec      VarUI4FromDec
#define VarUintFromInt      VarUI4FromI4

/* Mac Note: On the Mac, the coersion functions support the
 * Symantec C++ calling convention for float/double. To support
 * float/double arguments compiled with the MPW C compiler,
 * use the following APIs to move MPW float/double values into
 * a VARIANT.
 */

/*---------------------------------------------------------------------*/
/*            New VARIANT <-> string parsing functions                 */
/*---------------------------------------------------------------------*/

typedef struct {
    INT   cDig;
    ULONG dwInFlags;
    ULONG dwOutFlags;
    INT   cchUsed;
    INT   nBaseShift;
    INT   nPwr10;
} NUMPARSE;

/* flags used by both dwInFlags and dwOutFlags:
 */
#define NUMPRS_LEADING_WHITE    0x0001
#define NUMPRS_TRAILING_WHITE   0x0002
#define NUMPRS_LEADING_PLUS     0x0004
#define NUMPRS_TRAILING_PLUS    0x0008
#define NUMPRS_LEADING_MINUS    0x0010
#define NUMPRS_TRAILING_MINUS   0x0020
#define NUMPRS_HEX_OCT          0x0040
#define NUMPRS_PARENS           0x0080
#define NUMPRS_DECIMAL          0x0100
#define NUMPRS_THOUSANDS        0x0200
#define NUMPRS_CURRENCY         0x0400
#define NUMPRS_EXPONENT         0x0800
#define NUMPRS_USE_ALL          0x1000
#define NUMPRS_STD              0x1FFF

/* flags used by dwOutFlags only:
 */
#define NUMPRS_NEG              0x10000
#define NUMPRS_INEXACT          0x20000

/* flags used by VarNumFromParseNum to indicate acceptable result types:
 */
#define VTBIT_I1        (1 << VT_I1)
#define VTBIT_UI1       (1 << VT_UI1)
#define VTBIT_I2        (1 << VT_I2)
#define VTBIT_UI2       (1 << VT_UI2)
#define VTBIT_I4        (1 << VT_I4)
#define VTBIT_UI4       (1 << VT_UI4)
#define VTBIT_R4        (1 << VT_R4)
#define VTBIT_R8        (1 << VT_R8)
#define VTBIT_CY        (1 << VT_CY)
#define VTBIT_DECIMAL   (1 << VT_DECIMAL)


STDAPI VarParseNumFromStr(OLECHAR * strIn, LCID lcid, ULONG dwFlags,
            NUMPARSE * pnumprs, BYTE * rgbDig);

STDAPI VarNumFromParseNum(NUMPARSE * pnumprs, BYTE * rgbDig,
            ULONG dwVtBits, VARIANT * pvar);


/*---------------------------------------------------------------------*/
/*                   New date functions                                */
/*---------------------------------------------------------------------*/

#define VAR_VALIDDATE       0x0004    /* VarDateFromUdate() only */
#define VAR_CALENDAR_HIJRI  0x0008    /* use Hijri calender */
#define VARIANT_CALENDAR_HIJRI VAR_CALENDAR_HIJRI

/* The UDATE structure is used with VarDateFromUdate() and VarUdateFromDate().
 * It represents an "unpacked date".
 */
typedef struct {
    SYSTEMTIME st;
    USHORT  wDayOfYear;
} UDATE;

/* APIs to "pack" and "unpack" dates.
 */
STDAPI VarDateFromUdate(UDATE *pudateIn, ULONG dwFlags, DATE *pdateOut);

STDAPI VarUdateFromDate(DATE dateIn, ULONG dwFlags, UDATE *pudateOut);

/* API to retrieve the secondary(altername) month names
   Useful for Hijri, Polish and Russian alternate month names
*/   
STDAPI GetAltMonthNames(LCID lcid, LPOLESTR * * prgp);



/*---------------------------------------------------------------------*/
/*                 ITypeLib                                            */
/*---------------------------------------------------------------------*/

typedef ITypeLib * LPTYPELIB;


/*---------------------------------------------------------------------*/
/*                ITypeInfo                                            */
/*---------------------------------------------------------------------*/


typedef LONG DISPID;
typedef DISPID MEMBERID;

#define MEMBERID_NIL DISPID_UNKNOWN
#define ID_DEFAULTINST  -2


/* Flags for IDispatch::Invoke */
#define DISPATCH_METHOD         0x1
#define DISPATCH_PROPERTYGET    0x2
#define DISPATCH_PROPERTYPUT    0x4
#define DISPATCH_PROPERTYPUTREF 0x8

typedef ITypeInfo * LPTYPEINFO;


/*---------------------------------------------------------------------*/
/*                ITypeComp                                            */
/*---------------------------------------------------------------------*/

typedef ITypeComp * LPTYPECOMP;


/*---------------------------------------------------------------------*/
/*             ICreateTypeLib                                          */
/*---------------------------------------------------------------------*/

typedef ICreateTypeLib * LPCREATETYPELIB;

typedef ICreateTypeInfo * LPCREATETYPEINFO;

/*---------------------------------------------------------------------*/
/*             TypeInfo API                                            */
/*---------------------------------------------------------------------*/

/* compute a 16bit hash value for the given name
 */
#ifdef _WIN32
STDAPI_(ULONG) LHashValOfNameSysA(SYSKIND syskind, LCID lcid,
            LPCSTR szName);
#endif

STDAPI_(ULONG)
LHashValOfNameSys(SYSKIND syskind, LCID lcid, const OLECHAR * szName);

#define LHashValOfName(lcid, szName) \
            LHashValOfNameSys(SYS_WIN32, lcid, szName)

#define WHashValOfLHashVal(lhashval) \
            ((USHORT) (0x0000ffff & (lhashval)))

#define IsHashValCompatible(lhashval1, lhashval2) \
            ((BOOL) ((0x00ff0000 & (lhashval1)) == (0x00ff0000 & (lhashval2))))

/* load the typelib from the file with the given filename
 */
STDAPI LoadTypeLib(const OLECHAR  *szFile, ITypeLib ** pptlib);

/* Control how a type library is registered
 */
typedef enum tagREGKIND
{
    REGKIND_DEFAULT,
    REGKIND_REGISTER,
    REGKIND_NONE
} REGKIND;

STDAPI LoadTypeLibEx(LPCOLESTR szFile, REGKIND regkind,
            ITypeLib ** pptlib);

/* load registered typelib
 */
STDAPI LoadRegTypeLib(REFGUID rguid, WORD wVerMajor, WORD wVerMinor,
            LCID lcid, ITypeLib ** pptlib);

/* get path to registered typelib
 */
STDAPI QueryPathOfRegTypeLib(REFGUID guid, USHORT wMaj, USHORT wMin,
            LCID lcid, LPBSTR lpbstrPathName);

/* add typelib to registry
 */
STDAPI RegisterTypeLib(ITypeLib * ptlib, OLECHAR  *szFullPath,
            OLECHAR  *szHelpDir);

/* remove typelib from registry
 */
STDAPI DeregisterTypeLib(REFGUID rguid, WORD wVerMajor,
            WORD wVerMinor, LCID lcid);

STDAPI UnRegisterTypeLib(REFGUID libID, WORD wVerMajor,
            WORD wVerMinor, LCID lcid, SYSKIND syskind);

STDAPI CreateTypeLib(SYSKIND syskind, const OLECHAR  *szFile,
            ICreateTypeLib ** ppctlib);

STDAPI CreateTypeLib2(SYSKIND syskind, LPCOLESTR szFile,
            ICreateTypeLib2 **ppctlib);

STDAPI GetLocalHelpFileName(LPCOLESTR szHelpFileName,
            BSTR * pbstrLocalHelpFileName);


/*---------------------------------------------------------------------*/
/*           IDispatch implementation support                          */
/*---------------------------------------------------------------------*/

typedef IDispatch * LPDISPATCH;

typedef struct tagPARAMDATA {
    OLECHAR * szName;   /* parameter name */
    VARTYPE vt;         /* parameter type */
} PARAMDATA, * LPPARAMDATA;

typedef struct tagMETHODDATA {
    OLECHAR * szName;   /* method name */
    PARAMDATA * ppdata; /* pointer to an array of PARAMDATAs */
    DISPID dispid;      /* method ID */
    UINT iMeth;         /* method index */
    CALLCONV cc;        /* calling convention */
    UINT cArgs;         /* count of arguments */
    WORD wFlags;        /* same wFlags as on IDispatch::Invoke() */
    VARTYPE vtReturn;
} METHODDATA, * LPMETHODDATA;

typedef struct tagINTERFACEDATA {
    METHODDATA * pmethdata;  /* pointer to an array of METHODDATAs */
    UINT cMembers;      /* count of members */
} INTERFACEDATA, * LPINTERFACEDATA;



/* Locate the parameter indicated by the given position, and
 * return it coerced to the given target VARTYPE (vtTarg).
 */
STDAPI DispGetParam(DISPPARAMS * pdispparams, UINT position,
            VARTYPE vtTarg, VARIANT * pvarResult, UINT * puArgErr);

/* Automatic TypeInfo driven implementation of IDispatch::GetIDsOfNames()
 */
STDAPI DispGetIDsOfNames(ITypeInfo * ptinfo, OLECHAR ** rgszNames,
            UINT cNames, DISPID * rgdispid);

/* Automatic TypeInfo driven implementation of IDispatch::Invoke()
 */
STDAPI DispInvoke(void * _this, ITypeInfo * ptinfo, DISPID dispidMember,
            WORD wFlags, DISPPARAMS * pparams, VARIANT * pvarResult,
            EXCEPINFO * pexcepinfo, UINT * puArgErr);

/* Construct a TypeInfo from an interface data description
 */
STDAPI CreateDispTypeInfo(INTERFACEDATA * pidata, LCID lcid,
            ITypeInfo ** pptinfo);

/* Create an instance of the standard TypeInfo driven IDispatch
 * implementation.
 */
STDAPI CreateStdDispatch(IUnknown * punkOuter, void * pvThis,
            ITypeInfo * ptinfo, IUnknown ** ppunkStdDisp);

/* Low-level helper for IDispatch::Invoke() provides machine independence
 * for customized Invoke().
 */
STDAPI DispCallFunc(void * pvInstance, ULONG oVft, CALLCONV cc,
            VARTYPE vtReturn, UINT  cActuals, VARTYPE * prgvt,
            VARIANTARG ** prgpvarg, VARIANT * pvargResult);


/*---------------------------------------------------------------------*/
/*            Active Object Registration API                           */
/*---------------------------------------------------------------------*/

/* flags for RegisterActiveObject */
#define ACTIVEOBJECT_STRONG 0x0
#define ACTIVEOBJECT_WEAK 0x1

STDAPI RegisterActiveObject(IUnknown * punk, REFCLSID rclsid,
            DWORD dwFlags, DWORD * pdwRegister);

STDAPI RevokeActiveObject(DWORD dwRegister, void * pvReserved);

STDAPI GetActiveObject(REFCLSID rclsid, void * pvReserved,
            IUnknown ** ppunk);

/*---------------------------------------------------------------------*/
/*                           ErrorInfo API                             */
/*---------------------------------------------------------------------*/

STDAPI SetErrorInfo(ULONG dwReserved, IErrorInfo * perrinfo);
STDAPI GetErrorInfo(ULONG dwReserved, IErrorInfo ** pperrinfo);
STDAPI CreateErrorInfo(ICreateErrorInfo ** pperrinfo);

/*---------------------------------------------------------------------*/
/*                           MISC API                                  */
/*---------------------------------------------------------------------*/

STDAPI_(ULONG) OaBuildVersion(void);

// Declare variant access functions.

#if __STDC__ || defined(NONAMELESSUNION)
#define V_UNION(X, Y)   ((X)->n1.n2.n3.Y)
#define V_VT(X)         ((X)->n1.n2.vt)
#else
#define V_UNION(X, Y)   ((X)->Y)
#define V_VT(X)         ((X)->vt)
#endif

/* Variant access macros
 */
#define V_ISBYREF(X)     (V_VT(X)&VT_BYREF)
#define V_ISARRAY(X)     (V_VT(X)&VT_ARRAY)
#define V_ISVECTOR(X)    (V_VT(X)&VT_VECTOR)
#define V_NONE(X)        V_I2(X)

#define V_UI1(X)         V_UNION(X, bVal)
#define V_UI1REF(X)      V_UNION(X, pbVal)
#define V_I2(X)          V_UNION(X, iVal)
#define V_I2REF(X)       V_UNION(X, piVal)
#define V_I4(X)          V_UNION(X, lVal)
#define V_I4REF(X)       V_UNION(X, plVal)
#define V_R4(X)          V_UNION(X, fltVal)
#define V_R4REF(X)       V_UNION(X, pfltVal)
#define V_R8(X)          V_UNION(X, dblVal)
#define V_R8REF(X)       V_UNION(X, pdblVal)
#define V_I1(X)          V_UNION(X, cVal)
#define V_I1REF(X)       V_UNION(X, pcVal)
#define V_UI2(X)         V_UNION(X, uiVal)
#define V_UI2REF(X)      V_UNION(X, puiVal)
#define V_UI4(X)         V_UNION(X, ulVal)
#define V_UI4REF(X)      V_UNION(X, pulVal)
#define V_INT(X)         V_UNION(X, intVal)
#define V_INTREF(X)      V_UNION(X, pintVal)
#define V_UINT(X)        V_UNION(X, uintVal)
#define V_UINTREF(X)     V_UNION(X, puintVal)
#define V_CY(X)          V_UNION(X, cyVal)
#define V_CYREF(X)       V_UNION(X, pcyVal)
#define V_DATE(X)        V_UNION(X, date)
#define V_DATEREF(X)     V_UNION(X, pdate)
#define V_BSTR(X)        V_UNION(X, bstrVal)
#define V_BSTRREF(X)     V_UNION(X, pbstrVal)
#define V_DISPATCH(X)    V_UNION(X, pdispVal)
#define V_DISPATCHREF(X) V_UNION(X, ppdispVal)
#define V_ERROR(X)       V_UNION(X, scode)
#define V_ERRORREF(X)    V_UNION(X, pscode)
#define V_BOOL(X)        V_UNION(X, boolVal)
#define V_BOOLREF(X)     V_UNION(X, pboolVal)
#define V_UNKNOWN(X)     V_UNION(X, punkVal)
#define V_UNKNOWNREF(X)  V_UNION(X, ppunkVal)
#define V_VARIANTREF(X)  V_UNION(X, pvarVal)
#define V_ARRAY(X)       V_UNION(X, parray)
#define V_ARRAYREF(X)    V_UNION(X, pparray)
#define V_BYREF(X)       V_UNION(X, byref)

#define V_DECIMAL(X)     V_UNION(X, decloVal)
#define V_DECIMALREF(X)  V_UNION(X, pdecloVal)

#ifndef RC_INVOKED
#include <poppack.h>
#endif // RC_INVOKED

#endif     // __OLEAUTO_H__
