//---------------------------------------------------------------------------
// Game.cpp
//---------------------------------------------------------------------------
// Sample spr program
//---------------------------------------------------------------------------
// (C) Copyright 1992-1996 by Microsoft Corporation.  All rights reserved.
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
//---------------------------------------------------------------------------

#include "Main.h"
#include "Game.h"
#include "App.h"
#include "Spr.h"
#include "Score.h"
#include "SproidsP.h"
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>


//---------------------------------------------------------------------------
// DEBUG info
//---------------------------------------------------------------------------
SZTHISFILE


//---------------------------------------------------------------------------
// Forward Declares
//---------------------------------------------------------------------------
int DisplayScriptError(HINSTANCE hinst, HWND hwndParent, LPCSTR pszError);


//---------------------------------------------------------------------------
// Image data
//---------------------------------------------------------------------------
#define ADDPIX(    id,idNext,x,y,cx,cy) {iimg##idNext, x, y, cx, cy},
#define ADDPIXLAST(id,idNext,x,y,cx,cy) {iimg##idNext, x, y, cx, cy}
#define ADDPIXIMG
IMG Game_img[] = {
#include "SproidsP.pix"
};
#undef ADDPIX
#undef ADDPIXLAST
#undef ADDPIXIMG


//---------------------------------------------------------------------------
// Various Globals
//---------------------------------------------------------------------------
CGame     *g_pgame         = NULL;
ITypeInfo *g_ptinfoClsGame = NULL;
ITypeInfo *g_ptinfoIntGame = NULL;


//---------------------------------------------------------------------------
// Cover for contstructor.  Use instead of new CGame!
//---------------------------------------------------------------------------
HRESULT CGame::s_hr = E_FAIL;

HRESULT CGame::CreateGame
(
  HINSTANCE hinst,
  HWND      hwndDlg,
  HWND      hwndPS,
  HWND      hwndStat,
  IOleComponentManager *pcm,
  CGame   **ppgameOut
)
{
  CGame  *pgame;
  HRESULT hr;

  *ppgameOut = NULL;
  pgame = new CGame(hinst, hwndDlg, hwndPS, hwndStat, pcm);
  if (!pgame)
    return E_OUTOFMEMORY;
  else if (pgame->s_hr)
    {
    delete pgame;
    return s_hr;
    }

  // Create dummy game GAME_csc sprite classes
  for (int i=0; i<GAME_csc; i++)
    CHECK(CSpriteClass::Create(pgame->m_pdisp, &pgame->m_rgpsc[i]));

// ##### BEGIN ACTIVEX SCRIPTING SUPPORT #####
  // Add the Game as a Named Item & load its code
  CHECK(pgame->m_ps->AddNamedItem(L"Game", SCRIPTITEM_ISVISIBLE | SCRIPTITEM_ISSOURCE | SCRIPTITEM_EXISTS));
  if (!g_pszCmdLine || !strlen(g_pszCmdLine))
    {
    MessageBox(g_papp->m_hwndDlg, "Usage: Sproids <file>, where <file> is a VB Script", "Sproids", MB_OK | MB_ICONEXCLAMATION);
    hr = E_FAIL;
    }
  else
    pgame->AddCode(g_pszCmdLine, L"Game");
// #####  END  ACTIVEX SCRIPTING SUPPORT #####

  if (hr)
    {
CleanUp:
    // Error. Delete the CGame we created
    delete pgame;
    return hr;
    }

// ##### BEGIN ACTIVEX SCRIPTING SUPPORT #####
  // Start the script running...
  hr = pgame->m_ps->SetScriptState(SCRIPTSTATE_CONNECTED);
  if (hr)
    {
    delete pgame;
    return hr;
    }
// #####  END  ACTIVEX SCRIPTING SUPPORT #####

  // Success
  *ppgameOut = pgame;
  return S_OK;
}


//---------------------------------------------------------------------------
// Constructor
//---------------------------------------------------------------------------
CGame::CGame
(
  HINSTANCE hinst,
  HWND      hwndDlg,
  HWND      hwndPS,
  HWND      hwndStat,
  IOleComponentManager *pcm
)
{
  INIT_SIGNATURE(SIG_Game);

  // Setup globals
  g_pgame       = this;
  s_hr          = S_OK;

  // Init members
  m_hinst        = hinst;
  m_hwndDlg      = hwndDlg;
  m_hwndPS       = hwndPS;
  m_hwndStat     = hwndStat;
  m_cref         = 1;
  m_fPaused      = FALSE;
  m_fShipDead    = FALSE;
  m_fShipRestart = FALSE;
  m_fGameOver    = TRUE;
  m_pcm          = pcm;
  m_ps           = NULL;
  m_psp          = NULL;
  VariantInit(&m_varTag);
  for (int i=0; i<GAME_csc; i++)
    m_rgpsc[i]   = NULL;
  m_pscore       = NULL;
  m_pdisp        = NULL;

  m_pdisp = new CDisplay(hinst, hwndPS, GetDC(hwndPS),
                         this->GetUnknown(), FireCollide,
                         sizeof(Game_img)/sizeof(IMG), Game_img, ID_BMP_SPRITES);
  if (!m_pdisp)
    {
    s_hr = E_OUTOFMEMORY;
    return;
    }

  s_hr = CScore::CreateScore(
                      m_hinst,       // hinst to load resources from
                      m_hwndDlg,     // hwnd on which to draw score
                      m_hwndPS,      // hwnd of PlaySurface
                      m_hwndStat,    // hwnd of Status bar
                      2500,          // scoreFirst1Up
                      2500,          // dscoreNext1Up
                      3,             // cshipStart;
                      ID_BMP_SHIP,   // idbmpShip;
                      ID_BMP_PLUS,   // idbmpPlus;
                      &m_pscore);
  if (s_hr)
    return;
  ASSERT(m_pscore, "Good hr, but m_pscore==NULL");

// ##### BEGIN ACTIVEX SCRIPTING SUPPORT #####
  // Create the VBScript Engine
  extern const CLSID CLSID_VBScript;
  s_hr = CoCreateInstance(CLSID_VBScript, NULL, CLSCTX_INPROC_SERVER, IID_IUnknown, (void **)&m_ps);
  if (s_hr)
    {
    MessageBox(g_papp->m_hwndDlg, "Creating the VB Script engine failed.  VBS.DLL is probably not correctly registered.", "Sproids", MB_OK | MB_ICONEXCLAMATION);
    return;
    }
  // Script Enginge must support IOleScriptParse for us to use it
  s_hr = m_ps->QueryInterface(IID_IOleScriptParse, (void **)&m_psp);
  if (s_hr)
    return;
  s_hr = m_ps->SetScriptSite(this);
  if (s_hr)
    return;
  // Find IPersistStreamInit to InitNew() the object:
  IPersistStreamInit *ppersist;
  s_hr = m_ps->QueryInterface(IID_IPersistStreamInit, (void **)&ppersist);
  if (s_hr)
    return;
  ppersist->InitNew();
  ppersist->Release();
// #####  END  ACTIVEX SCRIPTING SUPPORT #####

  // Success
  s_hr = S_OK;
}


//---------------------------------------------------------------------------
// Clean up allocated resources
//---------------------------------------------------------------------------
CGame::~CGame
(
  void
)
{
  CHECK_SIGNATURE(SIG_Game);
  this->Close();
  DESTROY_SIGNATURE(SIG_Game);
}


//---------------------------------------------------------------------------
// Clean up allocated resources
//---------------------------------------------------------------------------
void CGame::Close
(
  void
)
{
  CHECK_SIGNATURE(SIG_Game);

  KillTimer(m_hwndPS, 0);

  g_pgame = NULL;

  // Clear this, since it may be an object
  VariantClear(&m_varTag);

// ##### BEGIN ACTIVEX SCRIPTING SUPPORT #####
  // Release the language engine, since it may hold on to us
  if (m_psp)
    m_psp->Release();
  if (m_ps)
    {
    m_ps->Close();
    m_ps->Release();
    }
// #####  END  ACTIVEX SCRIPTING SUPPORT #####

  // Drop all sink's that have Advise()'d us, since they might keep us alive
  m_cp.Close();

  // Release all existing sprites
  if (m_pdisp)
    m_pdisp->DestroyAll();

  // Close & Release the SpriteClasses
  for (int i=0; i<GAME_csc; i++)
    if (m_rgpsc[i])
      {
      m_rgpsc[i]->Close();
      m_rgpsc[i]->Release();
      m_rgpsc[i] = NULL;
      }

  // Now destroy the Score & Display objects
  if (m_pscore)
    {
    delete m_pscore;
    m_pscore = NULL;
    }
  if (m_pdisp)
    {
    delete m_pdisp;
    m_pdisp = NULL;
    }
}


//***************************************************************************
// IUnknown Interface
//***************************************************************************

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::QueryInterface
(
  REFIID  iid,
  LPVOID* ppvObjOut
)
{
  if (!ppvObjOut)
    return E_INVALIDARG;

  *ppvObjOut = NULL;

  if (iid == IID_IUnknown)
    *ppvObjOut = this->GetUnknown();
  else if (iid == IID_IDispatch)
    *ppvObjOut = this->GetDispatch();
  else if (iid == IID_IGame)
    *ppvObjOut = (IGame *)this;
  else if (iid == IID_IConnectionPointContainer)
    *ppvObjOut = (IConnectionPointContainer *)this;
  else if (iid == IID_IOleScriptSite)
    *ppvObjOut = (IOleScriptSite *)this;
  else if (iid == IID_IProvideClassInfo)
    *ppvObjOut = (IProvideClassInfo *)this;

  if (*ppvObjOut)
    {
    this->AddRef();
    return S_OK;
    }

  return E_NOINTERFACE;
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP_(ULONG) CGame::AddRef
(
  void 
)
{
  return ++m_cref;
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP_(ULONG) CGame::Release
(
  void 
)
{
  ASSERT(m_cref, "bad m_cref");
  m_cref--;
  if (!m_cref)
    {
    m_cp.Close();     // Make sure nobody's holding on to us because we're holding their sink
    if (!m_cref && !m_cp.m_cref)
      {
      delete this;
      return 0;
      }
    }
  return m_cref;
}


//***************************************************************************
// IDispatch Interface
//***************************************************************************

//---------------------------------------------------------------------------
// Method needed by COleAuto, so it can implement IDispatch for us.
//---------------------------------------------------------------------------
void CGame::GetTypeLibInfo
(
  HINSTANCE    *phinstOut,
  const GUID  **pplibidOut, 
  SHORT        *pwMajLib, 
  SHORT        *pwMinLib,
  const CLSID **ppclsidOut, 
  const IID   **ppiidOut, 
  ITypeLib   ***ppptlOut
)
{
  *phinstOut  = g_hinst;
  *pplibidOut = &LIBID_SPROIDS;
  *pwMajLib   = 1;
  *pwMinLib   = 0;
  *ppclsidOut = &CLSID_Game;
  *ppiidOut   = &IID_IGame;
  *ppptlOut   = &g_ptlMain;
}


//***************************************************************************
// IProvideClassInfo Interface
//***************************************************************************

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::GetClassInfo
(
  ITypeInfo** pptinfoOut
)
{
  HRESULT hr = ((COleAuto *)this)->CheckTypeInfo(0, 0x0409);
  if (hr)
    return hr;
  *pptinfoOut = g_ptinfoClsGame;
  (*pptinfoOut)->AddRef();
  return S_OK;
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::GetGUID
(
  DWORD dwGuidKind,
  GUID* pGUID
)
{
  if (pGUID)
    return E_INVALIDARG;

  if (dwGuidKind != GUIDKIND_DEFAULT_SOURCE_DISP_IID)
    return E_INVALIDARG;

  *pGUID = DIID_IGameEvents;
  return S_OK;
}


//***************************************************************************
// IConnectionPointContainer Interface
//***************************************************************************

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::EnumConnectionPoints
(
  LPENUMCONNECTIONPOINTS* ppEnum
)
{
  return E_NOTIMPL;   // UNDONE: Implement this method
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::FindConnectionPoint
(
  REFIID             iid,
  LPCONNECTIONPOINT* ppCpOut
)
{
  if (!ppCpOut)
    return E_INVALIDARG;

  if (iid == DIID_IGameEvents || iid == IID_IDispatch)
    {
    *ppCpOut = &m_cp;
    (*ppCpOut)->AddRef();
    return S_OK;
    }
    
  return E_NOINTERFACE;
}


//***************************************************************************
// Embedded IConnectionPoint Class
//***************************************************************************

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
CGame::XCP::XCP
(
  void
)
{
  m_cref = 1;
  for (int i=0; i<GAME_cADVISE; i++)
    m_rgpdisp[i] = NULL;
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
void CGame::XCP::Close
(
  void
)
{
  for (int i=0; i<SC_cADVISE; i++)
    if (m_rgpdisp[i])
      {
      m_rgpdisp[i]->Release();
      m_rgpdisp[i] = NULL;
      }
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::XCP::QueryInterface
(
  REFIID  iid,
  LPVOID* ppvObjOut
)
{
  if (!ppvObjOut)
    return E_INVALIDARG;

  *ppvObjOut = NULL;

  if (iid == IID_IUnknown)
    *ppvObjOut = this->GetUnknown();
  else if (iid == IID_IConnectionPoint)
    *ppvObjOut = (IConnectionPoint *)this;

  if (*ppvObjOut)
    {
    this->AddRef();
    return S_OK;
    }

  return E_NOINTERFACE;
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP_(ULONG) CGame::XCP::AddRef
(
  void 
)
{
  return ++m_cref;
}


#define GAMEOFCP(pcp)  ((CGame *)((char *)pcp - (char *)&(((CGame *)0)->m_cp)))
//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP_(ULONG) CGame::XCP::Release
(
  void 
)
{
  CGame *pgame = GAMEOFCP(this);
  ASSERT(m_cref, "bad m_cref");
  m_cref--;
  if (!m_cref && !pgame->m_cref)
    {
    delete pgame;
    return 0;
    }
  return m_cref;
}


//***************************************************************************
// IConnectionPoint Interface
//***************************************************************************

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::XCP::GetConnectionInterface
(
  IID* piid
)
{
  if (!piid)
    return E_INVALIDARG;

  memcpy(piid, &DIID_ISpriteClassEvents, sizeof(IID));
  return S_OK;
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::XCP::GetConnectionPointContainer
(
  IConnectionPointContainer** ppCPC
)
{
  if (!ppCPC)
    return E_INVALIDARG;

  *ppCPC = (CGame *)((char *)this - (char *)&(((CGame *)0)->m_cp));
  (*ppCPC)->AddRef();
  return S_OK;
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::XCP::Advise
(
  IUnknown* punkSink,
  DWORD*    pdwCookie
)
{
  IDispatch  *psce = NULL;
  IDispatch **ppdisp;
  HRESULT     hr;

  if (!punkSink || !pdwCookie)
    return E_INVALIDARG;
  *pdwCookie = 0;

  // Look for empty slot
  for (ppdisp=m_rgpdisp; *ppdisp && ppdisp<&m_rgpdisp[GAME_cADVISE]; ppdisp++)
    ;
  // Did we find one?
  if (ppdisp >= &m_rgpdisp[GAME_cADVISE])
    return E_FAIL;          // UNDONE: Error?

  // Now see if sink supports correct interface
  hr = punkSink->QueryInterface(DIID_ISpriteClassEvents, (void **)&psce);
  if (hr == E_NOINTERFACE)
    {
    hr = punkSink->QueryInterface(IID_IDispatch, (void **)&psce);
    if (hr)
      return hr;
    }
  ASSERT(psce, "QI but no ptr");

  // Finish advise by stashing punkSink QI'd to our interface
  *ppdisp = psce;       // AddRef'd from QI
  *pdwCookie = (ppdisp - m_rgpdisp) + 1;

  return S_OK;
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::XCP::Unadvise
(
  DWORD dwCookie
)
{
  if (!dwCookie || dwCookie > GAME_cADVISE)
    return E_INVALIDARG;

  m_rgpdisp[dwCookie-1]->Release();
  m_rgpdisp[dwCookie-1] = NULL;
  return S_OK;
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::XCP::EnumConnections
(
  LPENUMCONNECTIONS* ppEnum
)
{
  return E_NOTIMPL;   // UNDONE: Implement this method
}


// ##### BEGIN ACTIVEX SCRIPTING SUPPORT #####
//***************************************************************************
// IOleScriptSite Interface
//***************************************************************************

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::GetLCID
(
  LCID *plcid
)
{
  return E_NOTIMPL;     // Use system settings
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::GetItemInfo
(
  LPCOLESTR   pstrName,
  DWORD       dwReturnMask,
  IUnknown**  ppunkItemOut,
  ITypeInfo** pptinfoOut
)
{
  HRESULT hr;

  if (dwReturnMask & SCRIPTINFO_ITYPEINFO)
    {
    if (!pptinfoOut)
      return E_INVALIDARG;
    *pptinfoOut = NULL;
    }

  if (dwReturnMask & SCRIPTINFO_IUNKNOWN)
    {
    if (!ppunkItemOut)
      return E_INVALIDARG;
    *ppunkItemOut = NULL;
    }

  if (!_wcsicmp(L"game", pstrName))
    {
    if (dwReturnMask & SCRIPTINFO_ITYPEINFO)
      {
      hr = ((COleAuto *)this)->CheckTypeInfo(0, 0x0409);
      if (hr)
        return hr;
      *pptinfoOut = g_ptinfoClsGame;
      g_ptinfoClsGame->AddRef();    // for *pptinfoOut
      }
    if (dwReturnMask & SCRIPTINFO_IUNKNOWN)
      {
      *ppunkItemOut = this->GetUnknown();
      this->AddRef();               // for *ppunkItemOut
      }
    return S_OK;
    }

  return TYPE_E_ELEMENTNOTFOUND;
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::GetDocVersionString
(
  BSTR *pszVersion
)
{
  return E_NOTIMPL;   // UNDONE: Implement this method
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::RequestItems
(
  void 
)
{
  return m_ps->AddNamedItem(L"Game", SCRIPTITEM_ISVISIBLE | SCRIPTITEM_ISSOURCE | SCRIPTITEM_EXISTS);
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::RequestTypeLibs
(
  void 
)
{
  return m_ps->AddTypeLib(LIBID_SPROIDS, 1, 0, 0);
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::GetComponentManager
(
  IOleComponentManager **ppcm
)
{
  // Obsolete. Should never reach this.
  FAIL("GetComponentManager() not expected");
  return E_NOTIMPL;
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::EnableModeless
(
  BOOL fEnable
)
{
  if (fEnable)
    m_pcm->FOnComponentExitState(g_papp->m_idcomp, cmgrstateModal, cmgrcontextAll, 0, NULL);
  else
    m_pcm->OnComponentEnterState(g_papp->m_idcomp, cmgrstateModal, cmgrcontextAll, 0, NULL, NULL);
  return S_OK;
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::OnScriptTerminate
(
  const VARIANT   *pvarResult,
  const EXCEPINFO *pexcepinfo
)
{
  // UNDONE: Put up error dlg here
  return S_OK;
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::OnStateChange
(
  SCRIPTSTATE ssScriptState
)
{
  // Don't care about notification
  return S_OK;
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::OnScriptError
(
  const EXCEPINFO *pexcepinfo
)
{
  // Display the error
  MAKE_ANSIPTR_FROMWIDE(pszError, pexcepinfo->bstrDescription);
  DisplayScriptError(g_hinst, m_hwndDlg, pszError);
  return S_OK;
}


//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
STDMETHODIMP CGame::CreateItem
(
  LPCOLESTR  pstrName,
  IUnknown** ppunkItemNew
)
{
  // Obsolete. Should never reach this.
  FAIL("CreateItem() not expected");
  return E_NOTIMPL;
}
// #####  END  ACTIVEX SCRIPTING SUPPORT #####


//***************************************************************************
// Misc WndProc-related Methods
//***************************************************************************

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
BOOL AppEvt_DlgProc
(
  HWND   hwnd,
  UINT   msg,
  WPARAM wp,
  LPARAM lp
)
{
  if (!g_pgame)
    return FALSE;

  return g_pgame->DlgProc(hwnd, msg, wp, lp);
}


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
BOOL CGame::DlgProc
(
  HWND   hwnd,
  UINT   msg,
  WPARAM wp,
  LPARAM lp
)
{
  switch (msg)
    {
    case WM_ERASEBKGND:
      m_pscore->Paint((HDC)wp);
      return TRUE;
    }
  return FALSE;
}


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
LRESULT AppEvt_PSWndProc
(
  HWND   hwnd,
  UINT   msg,
  WPARAM wp,
  LPARAM lp
)
{
  if (!g_pgame)
    return DefWindowProc(hwnd, msg, wp, lp);

  return g_pgame->PSWndProc(hwnd, msg, wp, lp);
}


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
LRESULT CGame::PSWndProc
(
  HWND   hwnd,
  UINT   msg,
  WPARAM wp,
  LPARAM lp
)
{
  switch (msg)
    {
    case WM_PAINT:
      if (m_pdisp)
        {
        if (wp)
          m_pdisp->Paint((HDC)wp);
        else
          {
          PAINTSTRUCT ps;

          BeginPaint(hwnd, &ps);
          m_pdisp->Paint(ps.hdc);
          EndPaint(hwnd, &ps);
          }
        return 1L;
        }
      break;

    case WM_SIZE:
      {
      RECT rect;

      GetClientRect(hwnd, &rect);
      m_pdisp->m_cx = rect.right;
      m_pdisp->m_cy = rect.bottom;
      break;
      }

    case WM_TIMER:
      {
      if (m_fGameOver)
        break;

      m_pdisp->Timer();

      if (m_fPaused)
        break;

      if (m_fShipRestart)
        {
        m_pdisp->Refresh();
        break;
        }

      if (!m_fShipDead)
        {
        static cRefresh = 0;

        CGame::FireTick();
        if (++cRefresh > 5)	      // What's a reasonable number?
          {
          m_pdisp->Refresh();
          cRefresh = 0;
          }
        }
      else
        {
        m_pscore->ShipKilled();
        InvalidateRect(m_hwndDlg, NULL, TRUE);
        if (m_pscore->GetCShip() <= 0)
          {
          this->GameOver();
          break;
          }

        m_fShipDead = FALSE;
        m_fShipRestart = TRUE;
        InvalidateRect(hwnd, NULL, TRUE);
        }
      break;
      }

    case WM_KEYDOWN:
      FireKeyDown(wp);
      break;

    case WM_CHAR:
      FireKeyPress(wp);
      break;

    case WM_KEYUP:
      FireKeyUp(wp);
      break;
    }

  return DefWindowProc(hwnd, msg, wp, lp);
}


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
LRESULT CALLBACK ScriptErrorDialog
(
  HWND   hwnd,
  UINT   msg,
  WPARAM wp,
  LPARAM lp
)
{
  LPCSTR pszError;

  switch (msg) 
    {
    case WM_INITDIALOG:
      pszError = (LPCSTR)lp;
      ShowWindow(hwnd, SW_HIDE);
      SendDlgItemMessage(hwnd, ID_CTL_ERROR, WM_SETFONT,
                         (WPARAM)GetStockObject(ANSI_FIXED_FONT), MAKELPARAM(FALSE,0));
      SendDlgItemMessage(hwnd, ID_CTL_ERROR, WM_SETTEXT, 0, (LPARAM)pszError);
      ShowWindow(hwnd, SW_SHOW);
      return (TRUE);

     case WM_COMMAND:
       switch(LOWORD(wp))
         {
         case IDOK:
         case IDCANCEL:
           goto Destroy;
         }
       break;
    }
  return FALSE;

Destroy:
  EndDialog(hwnd, 0);
  return TRUE;
}


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
int DisplayScriptError
(
  HINSTANCE hinst,
  HWND      hwndParent,
  LPCSTR    pszError
)
{
  int   result;
  UINT  len = lstrlen(pszError);
  UINT  newlen;
  UINT  newindex;
  UINT  i;
  LPSTR pszNewError;

  // Figure out how long the real string will be
  newlen = 0;
  for (i=0; i<len; i++)
    {
    CHAR c = pszError[i];

    if (c == '\n')
      newlen +=1;               // Convert \n to \r\n
    else if (c == '\r')
      {
      if (pszError[i+1] == '\n')
        {
        newlen++;               // Leave \r\n alone
        i++;
        }
      else
        newlen++;               // Convert \r to \r\n
      }
    newlen++;
    }

  // Create the real string:
  pszNewError = new char[newlen+1];
  if (!pszNewError)
    return -1;

  newindex = 0;
  for (i=0; i<len; i++)
    {
    CHAR c = pszError[i];

    if (c == '\n')
      pszNewError[newindex++] = '\r';             // Convert \n to \r\n
    else if (c == '\r')
      {
      if (pszError[i+1] == '\n')
        {
        pszNewError[newindex++] = pszError[i++];  // Leave \r\n alone
        i++;
        }
      else
        {
        pszNewError[newindex++] = c;              // Convert \r to \r\n
        c = '\n';
        }
      }
    pszNewError[newindex++] = c;
    }
  pszNewError[newindex] = '\0';

  // Create the dialog box
  g_pgame->m_pcm->OnComponentEnterState(g_papp->m_idcomp, cmgrstateModal, cmgrcontextAll, 0, NULL, NULL);
  g_pgame->Pause(1);
  result = DialogBoxParam(
	  hinst,                         // Handle to application instance
	  MAKEINTRESOURCE(ID_DLG_ERROR), // Identifies dialog box template name  
	  hwndParent,                    // Handle to owner window
	  (DLGPROC)ScriptErrorDialog,    // Pointer to dialog box procedure
	  (DWORD)pszNewError);           // Context
  g_pgame->m_pcm->FOnComponentExitState(g_papp->m_idcomp, cmgrstateModal, cmgrcontextAll, 0, NULL);
  SetFocus(g_pgame->m_hwndPS);

  delete[] pszNewError;

  return result;
}


//***************************************************************************
// Misc General Methods
//***************************************************************************

//---------------------------------------------------------------------------
// New level
//---------------------------------------------------------------------------
void CGame::NewLevel
(
  void
)
{
  g_pgame->m_pdisp->DestroyAll();
  m_pscore->NextLevel();
  m_fShipRestart = TRUE;
  InvalidateRect(m_hwndPS,  NULL, TRUE);
  InvalidateRect(m_hwndDlg, NULL, TRUE);
  CGame::FireNewLevel();
}


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void AppEvt_NewGame
(
  void
)
{
  g_pgame->NewGame();
}


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void CGame::NewGame
(
  void
)
{
  m_pdisp->DestroyAll();
  m_pscore->NewGame();
  m_fGameOver    = FALSE;
  m_fShipRestart = TRUE;
  CGame::FireNewGame();
}


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
BOOL AppEvt_Pause
(
  int pause
)
{
  return g_pgame->Pause(pause);
}


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
BOOL CGame::Pause
(
  int pause
)
{
  if (pause == -1)            // Toggle
    pause = !m_fPaused;

  switch (pause)
    {
    case 1:                   // Pause
      if (m_fGameOver)
        goto UnPause;
      KillTimer(m_hwndPS, 0);
      m_fPaused = TRUE;
      break;

    case 0:                   // UnPause
UnPause:
      SetTimer(m_hwndPS, 0, 40, NULL);
      m_fPaused = FALSE;
      break;
    }

  if (m_fPaused)
    m_pscore->SetStatusText("Game Paused.  Press F3 to Continue.");
  else
    m_pscore->SetStatusText("");

  return m_fPaused;
}


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
BOOL AppEvt_FQueryTerminate
(
  BOOL fPromptUser
)
{
  return g_pgame->FQueryTerminate(fPromptUser);
}


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
BOOL CGame::FQueryTerminate
(
  BOOL fPromptUser
)
{
  // UNDONE
  return TRUE;
}


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void CGame::GameOver
(
  void
)
{
  m_fGameOver = TRUE;
  InvalidateRect(m_hwndPS, NULL, TRUE);
  m_pscore->SetStatusText("Game Over.  Press F2 to Play a New Game.");
}


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void CGame::AddCode
(
  char     *pszFileName,
  LPCOLESTR pstrItemName
)
{
  struct _stat stat;
  size_t       cch;
  EXCEPINFO    ei;
  char        *pszT;
  FILE        *pfile;
  HRESULT      hr;

  // Get data associated with "stat.c"
  if (_stat(pszFileName, &stat))
    return;
  cch = stat.st_size;

  pszT = new char[cch+1];
  if (!pszT)
    return;
  memset(pszT, 0, cch);   // HACK: Since cch may be too large, zero-fill string

  pfile = fopen(pszFileName, "r");
  if (pfile)
    {
    fread(pszT, cch, 1, pfile);
    fclose(pfile);
    pszT[cch] = 0;
    MAKE_WIDEPTR_FROMANSI(pwszCode, pszT);
// ##### BEGIN ACTIVEX SCRIPTING SUPPORT #####
    hr = m_psp->ParseScriptText(pwszCode, pstrItemName, NULL, NULL, SCRIPTTEXT_KEEPDEFINITIONS, NULL, &ei);
// #####  END  ACTIVEX SCRIPTING SUPPORT #####
    }
  delete pszT;
}

//--- EOF -------------------------------------------------------------------
