//---------------------------------------------------------------------------
// Game.h
//---------------------------------------------------------------------------
// Sample spr program
//---------------------------------------------------------------------------
// (C) Copyright 1992-1996 by Microsoft Corporation.  All rights reserved.
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Forward Delcares
//---------------------------------------------------------------------------
class CDisplay;
class CSpriteClass;
class CSprite;
class CScore;


//---------------------------------------------------------------------------
// the CGame class
//---------------------------------------------------------------------------
class CGame : public IGame, public COleAuto, 
              public IConnectionPointContainer, public IOleScriptSite,
              public IProvideClassInfo
  {
public:
  static HRESULT CGame::CreateGame(HINSTANCE hinst, HWND hwnd, HWND hwndPS, HWND hwndStat,
                                   IOleComponentManager *pcm, CGame **ppgameOut);
  CGame(HINSTANCE hinst, HWND hwnd, HWND hwndPS, HWND hwndStat, IOleComponentManager *pcm);
  ~CGame();
  void Close(void);

  // *** IUnknown methods ***
  STDMETHOD(QueryInterface)(REFIID riid, void** ppvObj);
  STDMETHOD_(ULONG, AddRef)(void);
  STDMETHOD_(ULONG, Release)(void);
  inline IUnknown  *GetUnknown(void)  {return (IGame *)this;};
  inline IDispatch *GetDispatch(void) {return (IGame *)this;};

  // *** IDispatch methods ***
  DECLARE_STANDARD_DISPATCH();
  // Override required by COleAuto:
  virtual ITypeInfo **GetTinfoClsAddr(void) { return &g_ptinfoClsGame; };
  virtual ITypeInfo **GetTinfoIntAddr(void) { return &g_ptinfoIntGame; };
  virtual void GetTypeLibInfo(HINSTANCE *phinstOut, const GUID **pplibidOut, 
                              SHORT *pwMajLibOut, SHORT *pwMinLibOut,
                              const CLSID **ppclsidOut, const IID **ppiidOut,
                              ITypeLib ***ppptlOut);
  virtual IDispatch *GetPrimary() {return this->GetDispatch(); };

  // *** IGame methods ***
  STDMETHOD(get_Caption)(BSTR* lpbstrReturn);
  STDMETHOD(put_Caption)(BSTR bstr);
  STDMETHOD(get_Application)(ISproidsApp** lppaReturn);
  STDMETHOD(get_Parent)(ISproidsApp** lppaReturn);
  STDMETHOD(StartGame)(void);
  STDMETHOD(EndGame)(void);
  STDMETHOD(RemoveAllSprites)(void);
  STDMETHOD(NextLevel)(void);
  STDMETHOD(Refresh)(void);
  STDMETHOD(AddScore)(int val);
  STDMETHOD(StdBorderBounce)(ISprite* pspr, int brd);
  STDMETHOD(StdBorderWrap)(  ISprite* pspr, int brd);
  STDMETHOD(StdInitRand)(ISprite* pspr, VARIANT user);
  STDMETHOD(StdInitEdge)(ISprite* pspr, VARIANT user);
  STDMETHOD(get_Paused)(VARIANT_BOOL* pRet);
  STDMETHOD(put_Paused)(VARIANT_BOOL val);
  STDMETHOD(get_Width)(int* pRet);
  STDMETHOD(put_Width)(int val);
  STDMETHOD(get_Height)(int* pRet);
  STDMETHOD(put_Height)(int val);
  STDMETHOD(get_Score)(int* pRet);
  STDMETHOD(put_Score)(int val);
  STDMETHOD(get_Level)(int* pRet);
  STDMETHOD(put_Level)(int val);
  STDMETHOD(get_ShipCount)(int* pRet);
  STDMETHOD(put_ShipCount)(int val);
  STDMETHOD(get_ScoreFirst1Up)(int* pRet);
  STDMETHOD(put_ScoreFirst1Up)(int val);
  STDMETHOD(get_DScoreNext1Up)(int* pRet);
  STDMETHOD(put_DScoreNext1Up)(int val);
  STDMETHOD(get_ShipsStart)(int* pRet);
  STDMETHOD(put_ShipsStart)(int val);
  STDMETHOD(get_Tag)(VARIANT* pRet);
  STDMETHOD(put_Tag)(VARIANT val);
  STDMETHOD(get_ShipClass)(ISpriteClass** pRet);
  STDMETHOD(get_BubbleClass)(ISpriteClass** pRet);
  STDMETHOD(get_AmmoClass)(ISpriteClass** pRet);
  STDMETHOD(get_ExtraClass)(ISpriteClass** pRet);

// ##### BEGIN ACTIVEX SCRIPTING SUPPORT #####
  // *** IOleScriptSite methods ***
  STDMETHOD(GetLCID)(LCID *plcid);
  STDMETHOD(GetItemInfo)(LPCOLESTR pstrName, DWORD dwReturnMask, IUnknown **ppiunkItem, ITypeInfo **ppti);
  STDMETHOD(GetDocVersionString)(BSTR *pszVersion);
  STDMETHOD(RequestItems)(void);
  STDMETHOD(RequestTypeLibs)(void);
  STDMETHOD(GetComponentManager)(IOleComponentManager **ppicm);
  STDMETHOD(EnableModeless)(BOOL fEnable);
  STDMETHOD(OnScriptTerminate)(const VARIANT *pvarResult, const EXCEPINFO *pexcepinfo);
  STDMETHOD(OnStateChange)(SCRIPTSTATE ssScriptState);
  STDMETHOD(OnScriptError)(const EXCEPINFO *pexcepinfo);
  STDMETHOD(CreateItem)(LPCOLESTR pstrName, IUnknown** ppunkItemNew);
// #####  END  ACTIVEX SCRIPTING SUPPORT #####

  // *** IProvideClassInfo methods ***
  STDMETHOD(GetClassInfo)(ITypeInfo** pptinfoOut);
  // *** IProvideClassInfo2 methods ***
  STDMETHOD(GetGUID)(DWORD dwGuidKind, GUID* pGUID);

  // *** IConnectionPointContainer methods ***
  STDMETHOD(EnumConnectionPoints)(LPENUMCONNECTIONPOINTS* ppEnum);
  STDMETHOD(FindConnectionPoint)(REFIID iid, LPCONNECTIONPOINT* ppCP);

  class XCP : public IConnectionPoint
    {
    public:
    XCP();
    void Close(void);

    // *** IUnknown methods ***
    STDMETHOD(QueryInterface)(REFIID iid, void** ppvObjOut);
    STDMETHOD_(ULONG, AddRef)(void);
    STDMETHOD_(ULONG, Release)(void);
    inline IUnknown *GetUnknown(void) {return (IConnectionPoint *)this;};

    // *** IConnectionPoint methods ***
    STDMETHOD(GetConnectionInterface)(IID* piid);
    STDMETHOD(GetConnectionPointContainer)(IConnectionPointContainer** ppCPC);
    STDMETHOD(Advise)(IUnknown* punkSink, DWORD* pdwCookie);
    STDMETHOD(Unadvise)(DWORD dwCookie);
    STDMETHOD(EnumConnections)(LPENUMCONNECTIONS* ppEnum);

    // *** Member Variables ***
    ULONG     m_cref;
#define GAME_cADVISE 5
    IDispatch *m_rgpdisp[GAME_cADVISE];     // Array of pdisps to call.
    } m_cp;

  // *** General Methods ***
  BOOL    DlgProc(  HWND hwnd, UINT msg, WPARAM wp, LPARAM lp);
  LRESULT PSWndProc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp);
  void    NewLevel(void);
  void    NewGame(void);
  BOOL    Pause(int pause);
  BOOL    FQueryTerminate(BOOL fPromptUser);
  void    GameOver(void);
  void    AddCode(char *pszFileName, LPCOLESTR pstrItemName);

  // *** Event Methods ***
  static void FireNewLevel(void);
  static void FireKeyDown(int vk);
  static void FireKeyPress(int ascii);
  static void FireKeyUp(int vk);
  static void FireTick(void);
  static void FireNewGame(void);
  static void FireCollide(CSprite *psprMoved, CSprite *psprHit, int);

  // *** Member Variables ***
  HINSTANCE  m_hinst;
  HWND       m_hwndDlg;
  HWND       m_hwndPS;
  HWND       m_hwndStat;
  CScore    *m_pscore;
  CDisplay  *m_pdisp;
  UINT       m_cref;
  VARIANT    m_varTag;
  IOleComponentManager *m_pcm;
// ##### BEGIN ACTIVEX SCRIPTING SUPPORT #####
  IOleScript           *m_ps;
  IOleScriptParse      *m_psp;
// #####  END  ACTIVEX SCRIPTING SUPPORT #####

  // The 3 SpriteClasses which make up our game
  #define GAME_csc    4
  CSpriteClass *m_rgpsc[GAME_csc];

  // These reset every new game
  BOOL m_fPaused:1;
  BOOL m_fShipDead:1;
  BOOL m_fShipRestart:1;
  BOOL m_fGameOver:1;

  // DEBUG info
  static HRESULT s_hr;
  #define SIG_Game 'Game'
  DECLARE_SIGNATURE(SIG_Game);
  };

//--- EOF -------------------------------------------------------------------
