/*****************************************************************/
/**               Microsoft Windows 4.00                        **/
/**           Copyright (C) Microsoft Corp., 1995-1996          **/
/*****************************************************************/

/* SITE.CPP - Methods for SITE class
 *
 * History:
 *  02/20/96    vlads   Created
 *
 */

#include "wbprov.h"

WPPSite::WPPSite(PCSTR pszFriendlyName,PCSTR pszURL)
    : m_cRef(1),
      m_fValid(FALSE),
      m_pDefaultSite(NULL)
{
    // Constructor logic
    m_fValid = TRUE;

	 // Get default site object
    //m_pDefaultSite = WPPCreateDefaultSite( pszFriendlyName, pszURL);
}

WPPSite::~WPPSite(void)
{

	 // If default pointer was obtained - release it
   	if (m_pDefaultSite) {
	 	m_pDefaultSite->Release();
	 }

    // Destructor and clean up
    m_fValid = FALSE;
}


//
// GLobal object generator function
//
dllexp
HRESULT
WppBindToSite(
	HWND	hwnd,
	LPCTSTR	lpszSiteName,
    LPCTSTR lpszURL,
    REFIID 	riid,
	DWORD	dwFlag,
	DWORD	dwReserved,
    PVOID * ppvObj)
{

    WPPSite *pSite = NULL;

    // Check requested interface ID
    if (!IsEqualIID(riid, IID_IUnknown) &&
        !IsEqualIID(riid, IID_IWPSite)) {
        return ResultFromScode(E_NOINTERFACE);
    }

	// This sample provider will handle URL 
	// http://wbprov
	if (!(lpszURL && !lstrcmpi(lpszURL, "http://wbprov")))
        return ResultFromScode(E_INVALIDARG);
		
#if 0
    // If it it my URL - create object and return pointer to it
    if (!lpszURL|| !*lpszURL) {
        return ResultFromScode(E_INVALIDARG);
    }
#endif

    // Create instance of object
    pSite = new WPPSite(NULL,lpszURL);

    // Was it sucessful ?
    if (!pSite || !pSite->IsValid()) {
        return ResultFromScode(E_OUTOFMEMORY);
    }

    *ppvObj = pSite;
    return NOERROR;
}


//
// *** IUnknown methods ***
//

STDMETHODIMP WPPSite::QueryInterface(REFIID riid, LPVOID * ppvObj)
{
    if (!IsEqualIID(riid, IID_IUnknown) &&
        !IsEqualIID(riid, IID_IWPSite)) {
        return ResultFromScode(E_NOINTERFACE);
    }

    *ppvObj = this;
    AddRef();
    return NOERROR;
}


STDMETHODIMP_(ULONG) WPPSite::AddRef(void)
{
    return ++m_cRef;
}


STDMETHODIMP_(ULONG) WPPSite::Release(void)
{
    ULONG cRef;

    cRef = --m_cRef;

    if (0L == m_cRef) {
        delete this;
    }

    return cRef;
}

//
// *** IWPSite methods ***
//

STDMETHODIMP WPPSite::NetworkConnect (LPCSTR lpszUserName,
						      LPCSTR lpszPassword)
{
	OutputDebugString("WBPROV: NetworkConnect");
	OutputDebugString("\r\n");
	return NOERROR;

#if 0
   if (m_pDefaultSite) {
	   return(m_pDefaultSite->NetworkConnect(lpszUserName,lpszPassword));
	}

   return ResultFromScode(E_NOTIMPL);
#endif
}

STDMETHODIMP WPPSite::NetworkDisconnect ()
{
	OutputDebugString("WBPROV: NetworkDisconnect");
	OutputDebugString("\r\n");
	return NOERROR;

#if 0
   if (m_pDefaultSite) {
	   return(m_pDefaultSite->NetworkDisconnect());
	}

    return ResultFromScode(E_NOTIMPL);
#endif
}

STDMETHODIMP WPPSite::ServerLogin (LPCSTR lpszUserName,
						      LPCSTR lpszPassword)
{
	OutputDebugString("WBPROV: ServerLogin");
	OutputDebugString("\r\n");
	return NOERROR;

#if 0
   if (m_pDefaultSite) {
	   return(m_pDefaultSite->ServerLogin(lpszUserName,lpszPassword));
	}

   return ResultFromScode(E_NOTIMPL);
#endif
}

STDMETHODIMP WPPSite::ServerLogout ()
{
	OutputDebugString("WBPROV: ServerLogout");
	OutputDebugString("\r\n");
	return NOERROR;

#if 0
   if (m_pDefaultSite) {
	   return(m_pDefaultSite->ServerLogout());
	}

   return ResultFromScode(E_NOTIMPL);
#endif
}


STDMETHODIMP WPPSite::GetSiteInfo (
						LPWPSITEINFO lpSiteInfo,
						LPDWORD lpcbSiteInfo)

{
	OutputDebugString("WBPROV: GetSiteInfo");
	OutputDebugString("\r\n");

   if (m_pDefaultSite) {
	   return(m_pDefaultSite->GetSiteInfo (lpSiteInfo,lpcbSiteInfo));
	}

   return ResultFromScode(E_NOTIMPL);
}

STDMETHODIMP WPPSite::PostFiles(DWORD 	cLocalPaths,
						LPSTR 	*lppszLocalPaths,
						LPDWORD	lpcbURL,
						LPSTR	lpszURL,
						DWORD 	dwFlag)

{
	OutputDebugString("WBPROV: PostFiles");
	OutputDebugString("\r\n");
	return NOERROR;

#if 0
   if (m_pDefaultSite) {
	   return(m_pDefaultSite->PostFiles(cLocalPaths,
                                       lppszLocalPaths,
                                       lpcbURL,
                                       lpszURL,
                                       dwFlag));
	}

   return ResultFromScode(E_NOTIMPL);
#endif
}

STDMETHODIMP WPPSite::Commit ()
{
	OutputDebugString("WBPROV: Commit");
	OutputDebugString("\r\n");

   if (m_pDefaultSite) {
	   return(m_pDefaultSite->Commit());
	}

    return ResultFromScode(E_NOTIMPL);
	
}

STDMETHODIMP WPPSite::GetParam(LPCSTR 	lpszParameter,
					       LPDWORD	lpcchValue,
					       LPSTR	lpszValue)

{
	OutputDebugString("WBPROV: GetParam");
	OutputDebugString("\r\n");

   if (m_pDefaultSite) {
	   return(m_pDefaultSite->GetParam(lpszParameter,
                                      lpcchValue,
                                      lpszValue));
	}

   return ResultFromScode(E_NOTIMPL);
	
}

STDMETHODIMP WPPSite::SetParam(LPCSTR	lpszParameter,
					       LPCSTR  lpszValue)

{
	OutputDebugString("WBPROV: SetParam");
	OutputDebugString("\r\n");

   if (m_pDefaultSite) {
	   return(m_pDefaultSite->SetParam(lpszParameter,
                                      lpszValue));
	}

    return ResultFromScode(E_NOTIMPL);
	
}

STDMETHODIMP WPPSite::GetError (
							LPDWORD	lpdwErrorType,
							LPDWORD	lpdwErrorCode,
							LPDWORD	lpcbError,
							LPTSTR	lpszError) 
{
	OutputDebugString("WBPROV: GetError");
	OutputDebugString("\r\n");

    return ResultFromScode(E_NOTIMPL);
}

#if 0
STDMETHODIMP WPPSite::AddWizardPages(LPVOID lpv,
						     LPFNADDPROPSHEETPAGE lpfnAdd,
						     LPARAM lParam)

{
    return ResultFromScode(E_NOTIMPL);
}
#endif
