/*****************************************************************/
/**               Microsoft Windows 4.00                        **/
/**           Copyright (C) Microsoft Corp., 1995-1996          **/
/*****************************************************************/

/* WPTEST.H - Test provider with COM interface
 *
 */

#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <regstr.h>

#include <wpapi.h>
#include  <wpspi.h>

//
// Important defines
//

#define ARRAYSIZE(a)    (sizeof(a)/sizeof(a[0]))

#define DATASEG_PERINSTANCE     ".instance"
#define DATASEG_SHARED          ".data"
#define DATASEG_READONLY         ".code"
#define DATASEG_DEFAULT     DATASEG_SHARED

#pragma data_seg(DATASEG_PERINSTANCE)

extern HANDLE hInstance;

// Set the default data segment
#pragma data_seg(DATASEG_DEFAULT)

//
// Module ref counting
//
extern UINT g_cRefThisDll;
extern UINT g_cLocks;

extern void LockThisDLL(BOOL fLock);
extern void RefThisDLL(BOOL fRef);

//
// Class definition for factory.
// Nb: This is not needed for first release, but will be needed as
// more COM functionality will be packaged inside provider
//
class WPPClassFactory : public IClassFactory
{
public:
    STDMETHODIMP QueryInterface(REFIID riid, LPVOID *ppv);
    STDMETHODIMP_(ULONG) AddRef(void);
    STDMETHODIMP_(ULONG) Release(void);

    STDMETHODIMP CreateInstance(
            /* [unique][in] */ IUnknown __RPC_FAR *pUnkOuter,
            /* [in] */ REFIID riid,
            /* [out] */ void __RPC_FAR *__RPC_FAR *ppvObject);

    STDMETHODIMP LockServer(
            /* [in] */ BOOL fLock);
};


//
// Class definition for provider object
// Nb: This is not needed for first release, but will be needed as
// more COM functionality will be packaged inside provider
//

class WPProvider : public IWPProvider
{

private:
    ULONG m_cRef;

    WPProvider();
    ~WPProvider();

public:
    // *** IUnknown methods ***
    STDMETHODIMP QueryInterface(REFIID riid, LPVOID * ppvObj);
    STDMETHODIMP_(ULONG) AddRef(void);
    STDMETHODIMP_(ULONG) Release(void);

    // *** Specific functions ***
    STDMETHOD(GetName) (THIS_ PSTR pbBuffer, PDWORD pcbBuffer) ;
    STDMETHOD(GetCapabilities) (THIS_ PDWORD pdwMask) PURE;
};

//
// Class definition for site object
//

class WPPSite : public IWPSite
{
friend dllexp HRESULT WppBindToSite(HWND 	hwnd,
									LPCTSTR lpszSiteName,
									LPCTSTR	lpszURL,
									REFIID  riid,
									DWORD	dwFlag,
									DWORD	dwReserved,
									PVOID 	*ppvObj);

private:
    ULONG   m_cRef;
    BOOL    m_fValid;

    WPPSite *m_pDefaultSite;


    WPPSite(PCSTR pszFriendlyName,PCSTR pszURL);
    ~WPPSite();

    BOOL inline IsValid(VOID) {
        return m_fValid;
    }

public:
    // *** IUnknown methods ***
    STDMETHODIMP QueryInterface(REFIID riid, LPVOID * ppvObj);
    STDMETHODIMP_(ULONG) AddRef(void);
    STDMETHODIMP_(ULONG) Release(void);

    // *** Specific functions ***
    STDMETHOD(GetSiteInfo) (THIS_
							LPWPSITEINFO lpSiteInfo,
							LPDWORD lpcbSiteInfo) ;
    STDMETHOD(NetworkConnect) (THIS_
								LPCSTR lpszUserName,
								LPCSTR lpszPassword) ;
    STDMETHOD(NetworkDisconnect) (THIS_ ) ;
    STDMETHOD(ServerLogin) (THIS_
							LPCSTR lpszUserName,
							LPCSTR lpszPassword);
    STDMETHOD(ServerLogout) (THIS_ ) ;
    STDMETHOD(PostFiles) (THIS_
							DWORD 	cLocalPaths,
							LPSTR 	*lppszLocalPaths,
							LPDWORD	lpcbURL,
							LPSTR	lpszURL,
							DWORD 	dwFlag) ;
    STDMETHOD(Commit) (THIS_ ) ;
    STDMETHOD(GetParam) (THIS_
							LPCSTR 	lpszParameter,
							LPDWORD	lpcchValue,
							LPSTR	lpszValue) ;
    STDMETHOD(SetParam) (THIS_
							LPCSTR	lpszParameter,
							LPCSTR	lpszValue) ;
	STDMETHOD(GetError) (THIS_
							LPDWORD	lpdwErrorType,
							LPDWORD	lpdwErrorCode,
							LPDWORD	lpcbError,
							LPTSTR	lpszError) ;
    STDMETHOD(AddWizardPages) (THIS_
								LPVOID lpv,
								LPFNADDPROPSHEETPAGE lpfnAdd,
								LPARAM lParam) ;


};


//
// Syncronization mechanisms
//
void Netlib_EnterCriticalSection(void);
void Netlib_LeaveCriticalSection(void);
#ifdef DEBUG
extern BOOL g_fCritical;
#endif
#define ENTERCRITICAL   Netlib_EnterCriticalSection();
#define LEAVECRITICAL   Netlib_LeaveCriticalSection();
#define ASSERTCRITICAL  ASSERT(g_fCritical);

